// Generated by CoffeeScript 1.12.4
(function() {
  var _, asterisk, components, decompose, hintRegex, normalize, parse, phoneUtil, resolve, stripRegex, stripType, supportedRegionCodes, tollFreeAreaCodes;

  _ = require('lodash');

  phoneUtil = require('libphonenumber').phoneUtil;

  normalize = require('../normalize');

  supportedRegionCodes = ['US', 'CA', 'GB'];

  tollFreeAreaCodes = {
    '800': true,
    '844': true,
    '855': true,
    '866': true,
    '877': true,
    '888': true
  };

  parse = function(string, req) {
    var mask, number, parts, raw, ref, ref1, ref2, regionCode, type;
    if (string == null) {
      return string;
    }
    ref = stripType(string), type = ref[0], string = ref[1];
    ref1 = resolve(string, req != null ? req.logger : void 0), number = ref1[0], regionCode = ref1[1], mask = ref1[2];
    raw = (ref2 = string.raw) != null ? ref2 : string;
    if (number) {
      parts = decompose(raw, number, regionCode, mask);
      parts.type = type;
      parts.valid = parts.masked || phoneUtil.isValidNumber(number);
      return parts;
    } else {
      parts = new String(string);
      parts.raw = raw;
      if (type != null) {
        parts.type = type;
      }
      parts.valid = false;
      return parts;
    }
  };

  hintRegex = /[(]?([hwcm])[)]?$/;

  stripRegex = /^\s+|\s+$/g;

  stripType = function(string) {
    var match, type;
    match = string.match(hintRegex);
    if (match) {
      type = (function() {
        switch (match[1]) {
          case 'h':
            return 'home';
          case 'w':
            return 'work';
          case 'c':
          case 'm':
            return 'mobile';
        }
      })();
      return [type, string.replace(hintRegex, '').replace(stripRegex, '')];
    } else {
      return [null, string];
    }
  };

  resolve = function(string, logger) {
    var e, i, len, mask, number, regionCode;
    mask = string.replace(/[^x\d\*]/g, '').split('').map(function(char) {
      return char === '*';
    });
    mask = mask.reverse();
    string = string.replace(/[*]/g, '0');
    number = null;
    for (i = 0, len = supportedRegionCodes.length; i < len; i++) {
      regionCode = supportedRegionCodes[i];
      try {
        number = phoneUtil.parse(string, regionCode);
      } catch (error) {
        e = error;
        if (logger != null) {
          if (typeof logger.error === "function") {
            logger.error(e);
          }
        }
        number = null;
      }
      if (number != null) {
        break;
      }
    }
    if (number) {
      return [number, regionCode, mask];
    } else {
      return [null, null, null];
    }
  };

  asterisk = function(str, mask) {
    var maskChar;
    maskChar = function(char, index) {
      if (mask[index]) {
        return '*';
      } else {
        return char;
      }
    };
    return str.split('').reverse().map(maskChar).reverse().join('');
  };

  decompose = function(raw, number, regionCode, mask) {
    var exchange, ext, extension, line, nationalDestinationCode, nationalDestinationCodeLength, nationalPrefix, nationalSignificantNumber, normal, phone, ref, subscriberNumber;
    nationalPrefix = phoneUtil.getNddPrefixForRegion(regionCode, true);
    nationalSignificantNumber = phoneUtil.getNationalSignificantNumber(number);
    nationalDestinationCodeLength = phoneUtil.getLengthOfNationalDestinationCode(number);
    if (nationalDestinationCodeLength > 0) {
      nationalDestinationCode = nationalSignificantNumber.substring(0, nationalDestinationCodeLength);
      subscriberNumber = nationalSignificantNumber.substring(nationalDestinationCodeLength);
    } else {
      nationalDestinationCode = null;
      subscriberNumber = nationalSignificantNumber.substring(nationalDestinationCodeLength);
    }
    extension = number.getExtension();
    if (mask) {
      ext = extension ? "x" + extension : '';
      if (nationalDestinationCode) {
        nationalDestinationCode = asterisk("" + nationalDestinationCode + subscriberNumber + ext, mask).split('x')[0].substring(0, nationalDestinationCode.length);
      }
      if (nationalSignificantNumber) {
        nationalSignificantNumber = asterisk("" + nationalSignificantNumber + ext, mask).split('x')[0];
      }
      if (subscriberNumber) {
        subscriberNumber = asterisk("" + subscriberNumber + ext, mask).split('x')[0];
      }
      if (extension) {
        extension = asterisk(ext, mask).replace(/x/, '');
      }
    }
    if (['US', 'CA'].indexOf(regionCode) !== -1) {
      exchange = subscriberNumber.substring(0, 3);
      line = subscriberNumber.substring(3);
    } else {
      null;
    }
    normal = nationalSignificantNumber ? _.compact([nationalSignificantNumber, extension]).join('x') : raw;
    phone = new String(normal);
    phone.prefix = nationalPrefix;
    phone.raw = raw;
    phone.area = nationalDestinationCode;
    phone.exchange = exchange;
    phone.line = line;
    phone.number = subscriberNumber;
    phone.extension = extension;
    phone.country_code = regionCode;
    if (mask.indexOf(true) >= 0) {
      phone.masked = true;
    }
    phone.is_tollfree = (ref = tollFreeAreaCodes[phone.area]) != null ? ref : false;
    return phone;
  };

  components = [
    {
      name: 'raw',
      type: 'string',
      description: 'Unmodified value'
    }, {
      name: 'area',
      type: 'string',
      description: 'Area code'
    }, {
      name: 'exchange',
      type: 'string',
      description: 'Exchange'
    }, {
      name: 'line',
      type: 'string',
      description: 'Line'
    }, {
      name: 'number',
      type: 'string',
      description: 'Full number'
    }, {
      name: 'extension',
      type: 'string',
      description: 'Extension'
    }, {
      name: 'country_code',
      type: 'string',
      description: 'Country code'
    }, {
      name: 'type',
      type: 'string',
      description: 'Number type: home, work, mobile'
    }, {
      name: 'is_tollfree',
      type: 'boolean',
      description: 'Whether the number is toll-free'
    }
  ];

  module.exports = {
    parse: parse,
    components: components,
    countryCodes: supportedRegionCodes,
    maskable: false,
    operators: ['is equal to', 'is not equal to', 'is blank', 'is not blank', 'format is valid', 'format is invalid', 'includes', 'does not include', 'is included in', 'is not included in'],
    examples: ['5127891111', '512 789-1111', '512-789-1111 x 44', '1 (512) 789-1111'].map(parse).map(normalize)
  };

}).call(this);
