// Generated by CoffeeScript 1.12.4
(function() {
  var Handlebars, components, handlers, named, normalize, parse, supportedRegionCodes;

  Handlebars = require('handlebars');

  named = require('named-regexp').named;

  normalize = require('../normalize');

  supportedRegionCodes = ['US', 'CA', 'GB'];

  handlers = {
    US: {
      regex: named(/^(:<code>(:<zip>\d{5})(?:\s*(?:\-)?\s*)?(:<four>\d{4})?)?$/i),
      format: '{{zip}}{{#if four}}-{{four}}{{/if}}'
    },
    CA: {
      regex: named(/^(:<code>(:<fsa>(:<district>[a-z])[0-9][a-z])\s*(:<ldu>[0-9][a-z][0-9]))$/i),
      format: '{{fsa}} {{ldu}}'
    },
    GB: {
      regex: named(/^(:<code>(:<outcode>[a-z]{1,2}[0-9]{1,2}[a-z0-9]{0,1})?\s*(:<incode>[0-9][a-z]{2}))$/i),
      format: '{{outcode}} {{incode}}'
    }
  };

  parse = function(string) {
    var captures, handler, i, key, len, match, normal, parsed, ref, ref1, ref2, region, value;
    for (i = 0, len = supportedRegionCodes.length; i < len; i++) {
      region = supportedRegionCodes[i];
      handler = handlers[region];
      match = handler.regex.exec(string.trim());
      if (match) {
        captures = match.captures;
        for (key in captures) {
          value = captures[key];
          captures[key] = value[0];
        }
        normal = Handlebars.compile(handler.format)(captures);
        parsed = new String(normal.toUpperCase());
        parsed.raw = (ref = string.raw) != null ? ref : string;
        parsed.country_code = region;
        for (key in captures) {
          value = captures[key];
          parsed[key] = (ref1 = value != null ? value.toUpperCase() : void 0) != null ? ref1 : null;
        }
        parsed.valid = true;
        return parsed;
      }
    }
    value = new String(string);
    value.raw = (ref2 = string.raw) != null ? ref2 : string;
    value.valid = false;
    return value;
  };

  components = [
    {
      name: 'raw',
      type: 'string',
      description: 'Unmodified value'
    }, {
      name: 'country_code',
      type: 'string',
      description: 'Country abbreviation'
    }, {
      name: 'zip',
      type: 'string',
      description: 'US zip code (first 5 digits)'
    }, {
      name: 'four',
      type: 'string',
      description: 'US zip code (last 4 digits)'
    }, {
      name: 'fsa',
      type: 'string',
      description: 'Canadian forward sortation area'
    }, {
      name: 'ldu',
      type: 'string',
      description: 'Canadian local delivery unit'
    }, {
      name: 'outcode',
      type: 'string',
      description: 'Great Britain outward code'
    }, {
      name: 'incode',
      type: 'string',
      description: 'Great Britain inward code'
    }
  ];

  module.exports = {
    parse: parse,
    components: components,
    maskable: false,
    operators: ['is equal to', 'is not equal to', 'is blank', 'is not blank', 'format is valid', 'format is invalid', 'includes', 'does not include', 'is included in', 'is not included in', 'matches pattern', 'does not match pattern'],
    examples: ['78751', '78751-4224', '78751 4224', 'Q2E 4U7', 'A11 1AA', 'AA11A 1AA'].map(parse).map(normalize)
  };

}).call(this);
