// Generated by CoffeeScript 1.12.4
(function() {
  var _, components, isoDateRegex, minRegex, named, normalize, number, parse, rangeRegex;

  _ = require('lodash');

  named = require('named-regexp').named;

  number = require('./number');

  normalize = require('../normalize');

  minRegex = named(/(:<min>-?\d+(?:\.\d+)?)\s*\+/);

  rangeRegex = named(/(:<min>-?\d+(?:\.\d+)?)\s+(:<max>\d+(?:\.\d+)?)/);

  isoDateRegex = /\d{4}-\d{2}-\d{2}(?:T\d{2}:\d{2}:\d{2}\.\d{1,3}Z)?/g;

  parse = function(string) {
    var captures, getTime, m, match, max, min, num, parsed, parts, raw, ref, ref1, ref2, sanitized;
    if (string == null) {
      return string;
    }
    if (string.replace == null) {
      string = string.toString();
    }
    raw = (ref = string.raw) != null ? ref : string;
    if (string.match(isoDateRegex)) {
      parts = [];
      while ((m = isoDateRegex.exec(string)) !== null) {
        parts.push(m[0]);
      }
      getTime = function(s) {
        return new Date(s).getTime().toString();
      };
      string = _.take(parts.map(getTime), 2).join('-');
    }
    sanitized = string.replace(/(.+)-|(?:to)/, '$1 ').replace(/[^-0-9 .+]/g, '');
    match = minRegex.exec(sanitized) || rangeRegex.exec(sanitized);
    if (match) {
      captures = match.captures;
      min = (((ref1 = captures.min) != null ? ref1[0] : void 0) != null) && parseFloat(captures.min[0]);
      max = (((ref2 = captures.max) != null ? ref2[0] : void 0) != null) && parseFloat(captures.max[0]);
      if (_(min).isNumber() && _(max).isNumber()) {
        parsed = new String(min + "-" + max);
        parsed.raw = raw;
        parsed.avg = parseFloat(((min + max) / 2).toFixed(2));
        parsed.mid = parseInt((min + max) / 2);
        parsed.min = min;
        parsed.max = max;
      } else if (_(min).isNumber()) {
        parsed = new String(min + "+");
        parsed.raw = raw;
        parsed.avg = null;
        parsed.mid = null;
        parsed.min = min;
        parsed.max = null;
      } else {
        parsed = new String(string);
        parsed.raw = string;
        parsed.valid = false;
      }
    } else {
      num = number.parse(string);
      parsed = new String(string);
      parsed.raw = raw;
      parsed.min = (num != null ? num.valid : void 0) ? num.valueOf() : null;
      parsed.max = parsed.min;
      parsed.avg = parsed.min;
      parsed.mid = parsed.min;
      parsed.valid = num.valid;
    }
    if (parsed.valid == null) {
      parsed.valid = true;
    }
    return parsed;
  };

  components = [
    {
      name: 'raw',
      type: 'string',
      description: 'Unmodified value'
    }, {
      name: 'mid',
      type: 'number',
      description: 'Average of max and min, rounded down to the nearest whole number'
    }, {
      name: 'max',
      type: 'number',
      description: 'Highest number in range'
    }, {
      name: 'min',
      type: 'number',
      description: 'Lowest number in range'
    }, {
      name: 'avg',
      type: 'number',
      description: 'Average of min and max'
    }
  ];

  module.exports = {
    parse: parse,
    components: components,
    maskable: false,
    operators: ['is equal to', 'is not equal to', 'is less than', 'is less than or equal to', 'is greater than', 'is greater than or equal to', 'is blank', 'is not blank', 'format is valid', 'format is invalid', 'includes', 'does not include', 'is included in', 'is not included in'],
    examples: ['100 to 200', '100 - 200', '100+', '100'].map(parse).map(normalize)
  };

}).call(this);
